// ********************************************************************
// SED1520, PT6520, AX6120, SBN1661 und kompatible
// Display-Beispiel: EA DIP-122 (www.lcd-module.de)
// Version 1.0 / Stand: 12.03.2010
// AVR-Version / GGC 4.3.2 / AVR-Studio 4.18
// Mikrocontroller: ATmega644
// Frequenz: 11.059.200 Hz
// ********************************************************************
// FLASH-Speicherbedarf:    ca. 7,7 kByte
// SRAM-Speicherbedarf:     ca. 0,2 kByte
// EEPROM-Speicherbedarf:   -
// ********************************************************************
// Die Programmfunktionen drfen fr private Nutzung verwendet und
// verndert werden. Fr gewerbliche Nutzung ist die Zustimmung
// der Authoren erforderlich.
// Kontakt: Matthias Kahnt, e-mail: az51@gmx.net
// ********************************************************************
// Die Nutzung erfolgt auf eigene Gefahr.
// Fr eventuell entstehende Schden wird keine Haftung bernommen.
// ********************************************************************
//
//************************************************************************
// AVR-Typ: ATmega644
//************************************************************************
#define F_CPU   11059200UL      // Taktfrequenz AVR

//************************************************************************
// Includebereich
//************************************************************************
#include <util/delay.h>         // Pausenfunktionen
#include <avr/io.h>             // IO-Grundfunktionen
#include <avr/interrupt.h>      // Interrupt-Funktionen
#include <avr/pgmspace.h>       // PROGMEM-Funktionen
#include <string.h>             // Stringfunktion
#include <stdio.h>              // Standardfunktion
#include <stdlib.h>             // Standardbibliothek

//************************************************************************
// DIP122 Belegung
//************************************************************************
//  ********************************************************
//  *  9   D2      PA2                     PA3     D3  10  *
//  *  8   D1      PA1                     PA4     D4  11  *
//  *  7   D0      PA0                     PA5     D5  12  *
//  *  6   E1      PB1                     PA6     D6  13  *
//  *  5   R/W     PB2                     PA7     D7  14  *
//  *  4   A0      PB3                     PD5     E2  15  *
//  *  3   VEE     -4V                     PD6     RES 16  *
//  *  2   VDD                                     A   17  *
//  *  1   VSS                                     K   18  *
//  ********************************************************

//************************************************************************
// Definebereich
//************************************************************************
#define LCD_A0_PIN      PB3
#define LCD_RW_PIN      PB2
#define LCD_E1_PIN      PB1
#define LCD_E2_PIN      PD5
#define LCD_RES_PIN     PD6
#define LCD_DATA_PORT   PORTA
#define LCD_DATA_READ   PINA
#define LCD_DATA_DDR    DDRA

#define LCD_A0_H    PORTB |=  (1<<LCD_A0_PIN)  /* LCD A0  */
#define LCD_A0_L    PORTB &= ~(1<<LCD_A0_PIN)

#define LCD_RW_H    PORTB |=  (1<<LCD_RW_PIN)  /* LCD RW  */
#define LCD_RW_L    PORTB &= ~(1<<LCD_RW_PIN)

#define LCD_E1_H    PORTB |=  (1<<LCD_E1_PIN)  /* LCD E1  */
#define LCD_E1_L    PORTB &= ~(1<<LCD_E1_PIN)

#define LCD_E2_H    PORTD |=  (1<<LCD_E2_PIN)  /* LCD E2  */
#define LCD_E2_L    PORTD &= ~(1<<LCD_E2_PIN)

#define LCD_RES_H   PORTD |=  (1<<LCD_RES_PIN) /* LCD RES */
#define LCD_RES_L   PORTD &= ~(1<<LCD_RES_PIN)

//************************************************************************
// Grundfunktionen
//************************************************************************
#include "AVR_SED1520_Funktionen.c"


// ***********************************************************************
// Grafik-Testprogramm
// ***********************************************************************
void grafik_test(void) {
unsigned char zs, x, y;
  draw_bitmap((char*)helloworldbmp,0,0,122,32,0); // Bitmapp 122x32 ausgeben
  _delay_ms(2000);  // Pause
  clear_lcd();      // Displayspeicher lschen
  printf_lcd(0,0,"Linien gerade:",1);           // Linien zeichnen
  draw_clear_line(0,10,121,10,1);
  draw_clear_line(5,14,115,14,1);
  draw_clear_line(10,18,110,18,1);
  draw_clear_line(15,22,105,22,1);
  draw_clear_line(20,26,100,26,1);  
  draw_clear_line(0,10,0,31,1);
  draw_clear_line(121,10,121,31,1);
  draw_clear_line(0,31,121,31,1);
  _delay_ms(2000);  // Pause
  clear_lcd();      // Displayspeicher lschen
  printf_lcd(0,0,"Linien diagonal:",1);         // Linien zeichnen
  draw_clear_line(0,10,121,31,1);
  draw_clear_line(0,31,121,10,1);
  _delay_ms(2000);  // Pause
  clear_lcd();      // Displayspeicher lschen
  printf_lcd(0,0,"Box ungef\x02llt:",1);        // ungefllte Box zeichnen
  draw_clear_box(0,10,121,31,1);
  draw_clear_box(2,12,119,29,1);
  draw_clear_box(4,14,117,27,1);
  draw_clear_box(6,16,115,25,1);  
  draw_clear_box(8,18,113,23,1);  
  _delay_ms(2000);  // Pause
  printf_lcd(0,0,"Box l\x03schen:",1);          // Box wieder lschen
  draw_clear_box(8,18,113,23,0);  
  draw_clear_box(6,16,115,25,0);  
  draw_clear_box(4,14,117,27,0);
  draw_clear_box(2,12,119,29,0);
  draw_clear_box(0,10,121,31,0);
  _delay_ms(2000);  // Pause
  clear_lcd();      // Displayspeicher lschen
  printf_lcd(0,0,"Box gef\x02llt:",1);          // gefllte Box zeichnen
  draw_clear_boxfilled(10,10,110,29,1);
  _delay_ms(2000);  // Pause
  clear_lcd();      // Displayspeicher lschen
  printf_lcd(0,0,"Kreise:",1);                  // Kreise zeichnen
  draw_clear_circle(15,20,8,1);
  draw_clear_circle(105,20,8,1);
  draw_clear_circle(60,20,8,1);
  draw_clear_circle(60,20,7,1);
  draw_clear_circle(60,20,6,1);
  draw_clear_circle(60,20,5,1);
  draw_clear_circle(60,20,4,1);
  draw_clear_circle(60,20,3,1);
  draw_clear_circle(60,20,2,1);    
  _delay_ms(2000);  // Pause
  clear_lcd();      // Displayspeicher lschen
  draw_bitmap((char*)cheersbmp,0,0,32,32,1);    // Bitmapp 32x32 ausgeben
  draw_bitmap((char*)footballbmp,45,0,32,32,0); // Bitmapp 32x32 ausgeben
  draw_bitmap((char*)flowersbmp,89,0,32,32,1);  // Bitmapp 32x32 ausgeben
  _delay_ms(2000);  // Pause
  printf_lcd(6,3,"-Bitmap-",0);                 // Text dazu ausgeben
  _delay_ms(2000);  // Pause
  clear_lcd();      // Displayspeicher lschen
  draw_bitmap((char*)test1bmp,0,0,122,32,0);    // Bitmapp 122x32 ausgeben
  _delay_ms(2000);  // Pause
  clear_lcd();      // Displayspeicher lschen
  printf_lcd(0,0,"LCD-Test:Zeichensatz",1);     // Zeichensatz ausgeben
  x = 0;
  y = 1;
  for(zs = 0x01; zs; zs++) {
    _delay_ms(50);      // Pause
    write_lcd_zeichen(x,y,zs);
    x++;
    if (x > 19) {
      _delay_ms(50);  // Pause
      x = 0;
      y++;
      if (y > 3) y = 1;
    }
  }
  _delay_ms(5000);  // Pause
  clear_lcd();      // Displayspeicher lschen
}


// ********************************************************************
// Hauptprogramm
// ********************************************************************
int main (void) {
  init_lcd();           // System und LCD initialisieren
  while(1) {
    printf_lcd     (0,0,"********************",1);
    printf_lcd_prop(0,1,"  AVR Grafik-LCD"    ,1);
    printf_lcd     (0,3,"********************",1);
    _delay_ms(2000);    // Pause
    clear_lcd();        // Displayspeicher lschen
    grafik_test();      // Grafikfunktionen testen
    printf_lcd_prop(0,0,"DIP122-5 122x32px",1);
    printf_lcd     (0,2,"SED1520 (2) Grafic",1);
    printf_lcd     (0,3,"V1 12.03.2010 ATmega",1);
    _delay_ms(2000);    // Pause
    clear_lcd();        // Displayspeicher lschen
  }
}

